@@ -1,4 +1,4 @@
-/* Copyright (c) 1993-2007 by Richard Kelsey and Jonathan Rees.
+/* Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
 #include <unistd.h>
@@ -12,6 +12,9 @@
 #include <string.h>		/* FD_ZERO sometimes needs this */
 #include <locale.h>		/* ISO C99 */
 #include "sysdep.h"
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
 #include "c-mods.h"
 #include "scheme48vm.h"
 #include "event.h"
@@ -91,6 +94,16 @@ psbool ps_check_fd(long fd_as_long, psbool is_read, long *status)
   int fd = (int)fd_as_long;
   int ready;
 
+#ifdef HAVE_POLL
+  struct pollfd pollfds[1];
+  pollfds[0].fd = fd;
+  pollfds[0].events = is_read? POLLIN : POLLOUT;
+
+  *status = NO_ERRORS;
+
+  while(1) {
+    ready = poll(pollfds, 1, 0);
+#else /* not HAVE_POLL */
   struct timeval timeout;
   fd_set fds;
 
@@ -106,6 +119,7 @@ psbool ps_check_fd(long fd_as_long, psbool is_read, long *status)
 		   is_read ? NULL : &fds,
 		   &fds,
 		   &timeout);
+#endif /* not HAVE_POLL */
     if (ready == 0)
 	return PSFALSE;
     /* Mike has witnessed it return 2 on Mac OS X. */
@@ -124,6 +138,19 @@ ps_read_fd(long fd_as_long, char *buffer, long max, psbool waitp,
   void *buf = (void *)buffer;
   int fd = (int)fd_as_long;
 
+#ifdef HAVE_POLL
+  struct pollfd pollfds[1];
+  pollfds[0].fd = fd;
+  pollfds[0].events = POLLIN;
+
+  /* for the normal return */
+  *eofp = PSFALSE;
+  *pending = PSFALSE;
+  *status = NO_ERRORS;
+
+  while(1) {
+    ready = poll(pollfds, 1, 0);
+#else /* not HAVE_POLL */
   struct timeval timeout;
   fd_set readfds;
 
@@ -138,6 +165,7 @@ ps_read_fd(long fd_as_long, char *buffer, long max, psbool waitp,
 
   while(1) {
     ready = select(fd + 1, &readfds, NULL, &readfds, &timeout);
+#endif /* not HAVE_POLL */
     if (ready == 0) {
       if (!waitp)
 	return 0;
