@@ -1,4 +1,4 @@
-/* Copyright (c) 1993-2007 by Richard Kelsey and Jonathan Rees.
+/* Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
 #define _WIN32_WINNT 0x0400 /* for QueueUserAPC */
@@ -331,7 +331,8 @@ static BOOL is_STD_INPUT_console,
   is_STD_OUTPUT_console,
   is_STD_ERROR_console;
 
-HANDLE s48_main_thread;
+/* defined and initialized in event.c */
+extern HANDLE s48_main_thread;
 
 DWORD WINAPI
 reader_thread_proc(LPVOID lpParameter)
@@ -480,22 +481,7 @@ writer_thread_proc(LPVOID lpParameter)
   return 0;
 }
 
-static HANDLE
-create_mutex_semaphore()
-{
-  HANDLE handle = CreateSemaphore(NULL, /* lpSemaphoreAttributes */
-				  0, /* lInitialCount */
-				  1, /* lMaximumCount */
-				  NULL); /* lpName */
-
-  if (handle == NULL)
-    {
-      fprintf(stderr, "error in CreateSemaphore\n");
-      exit(-1);
-    }
-
-  return handle;
-}
+extern HANDLE s48_create_mutex_semaphore();
 
 static void
 start_reader_slash_writer_thread(HANDLE file_handle,
@@ -514,8 +500,8 @@ start_reader_slash_writer_thread(HANDLE file_handle,
   thread_data->eof = PSFALSE;
   
   thread_data->file_handle = file_handle;
-  thread_data->check_semaphore = create_mutex_semaphore();
-  thread_data->abort_semaphore = create_mutex_semaphore();
+  thread_data->check_semaphore = s48_create_mutex_semaphore();
+  thread_data->abort_semaphore = s48_create_mutex_semaphore();
 
   thread_handle = CreateThread(NULL, /* lpThreadAttributes */
 			       4096, /* dwStackSize, */
@@ -571,7 +557,7 @@ ps_close_fd(long fd)
     case STREAM_FILE_REGULAR:
       {
 	HANDLE handle = stream_descriptor->file_regular_data.handle;
-	
+
 	if (CloseHandle(handle))
 	  status = NO_ERRORS;
 	else
@@ -594,7 +580,7 @@ ps_close_fd(long fd)
     case STREAM_SOCKET:
       {
 	SOCKET socket = stream_descriptor->socket_data.socket;
-	
+
 	if (closesocket(socket) == 0)
 	  status = NO_ERRORS;
 	else
@@ -957,7 +943,7 @@ ps_read_fd(long fd, char *buffer, long max, psbool waitp,
 
 	if ((wsa_status == 0)
 	    && (numberOfBytesRecvd == 0))
-	  *eofp = PSTRUE;
+	    *eofp = PSTRUE;
 
 	if ((wsa_status == 0)
 	    || (WSAGetLastError() == WSA_IO_PENDING))
@@ -1169,10 +1155,41 @@ long
 ps_abort_fd_op(long fd_as_long)
 {
   int fd = (int)fd_as_long;
+  stream_descriptor_t* stream_descriptor = &(stream_descriptors[fd]);
+  psbool pending_p = s48_is_pending(fd);
 
   if (!s48_remove_fd(fd))
     fprintf(stderr, "Error: ps_abort_fd_op, no pending operation on fd %d\n",
 	            fd);
+
+  switch (stream_descriptor->type)
+    {
+    case STREAM_FILE_REGULAR:
+      {
+	HANDLE handle = stream_descriptor->file_regular_data.handle;
+	CancelIo(handle);
+
+	/* flush any pending completion routines */
+	if (pending_p)
+	  while (SleepEx(0, TRUE) == IO_COMPLETION_EVENT)
+	    ;
+
+	break;
+      }
+    case STREAM_SOCKET:
+      {
+	SOCKET socket = stream_descriptor->socket_data.socket;
+	CancelIo((HANDLE) socket);
+
+	/* flush any pending completion routines */
+	if (pending_p)
+	  while (SleepEx(0, TRUE) == IO_COMPLETION_EVENT)
+	    ;
+
+	break;
+      }
+    }
+
   return 0;      /* because we do not actually do any I/O in parallel the
 		    status is always zero: no characters transfered. */
 }
@@ -1200,18 +1217,6 @@ s48_fd_io_init()
 
   initialize_stream_descriptors();
 
-  /* Yes, this is the official hoopla to get at an absolute handle for
-     the current thread.  GetCurrentThread() returns a *constant*. */
-     
-  if (!DuplicateHandle(GetCurrentProcess(),
-		       GetCurrentThread(), GetCurrentProcess(),
-		       &s48_main_thread,
-		       THREAD_ALL_ACCESS, FALSE, 0))
-    {
-      fprintf(stderr, "DuplicateHandle failed\n");
-      exit(-1);
-    }
-
   /* these Unix-style indices are hard-wired into the VM */
   handle = GetStdHandle(STD_INPUT_HANDLE);
   /*
@@ -1260,6 +1265,10 @@ static s48_value	s48_socket(s48_value udp_p, s48_value input_p),
                         s48_dup_socket_channel(s48_value socket_fd),
 			s48_close_socket_half(s48_value socket_channel,
 					      s48_value input_p),
+                        s48_get_host_by_name(s48_value machine),
+                        s48_get_host_by_name_result(s48_value event_uid),
+                        s48_get_host_by_address(s48_value address),
+                        s48_get_host_by_address_result(s48_value event_uid),
 			s48_get_host_name(void);
 
 /* Forward declaration. */
@@ -1292,6 +1301,10 @@ s48_init_socket(void)
   S48_EXPORT_FUNCTION(s48_connect);
   S48_EXPORT_FUNCTION(s48_dup_socket_channel);
   S48_EXPORT_FUNCTION(s48_close_socket_half);
+  S48_EXPORT_FUNCTION(s48_get_host_by_name);
+  S48_EXPORT_FUNCTION(s48_get_host_by_name_result);
+  S48_EXPORT_FUNCTION(s48_get_host_by_address);
+  S48_EXPORT_FUNCTION(s48_get_host_by_address_result);
   S48_EXPORT_FUNCTION(s48_get_host_name);
 }
 
@@ -1330,6 +1343,227 @@ raise_windows_error(DWORD id)
 #undef ERROR_BUFFER_SIZE
 }
 
+
+#define S48_MAX_HOST_NAME_LENGTH NI_MAXHOST
+
+struct gethostbyname_handshake
+{
+  long event_uid;
+  char host_name[S48_MAX_HOST_NAME_LENGTH + 1]; /* is enough */
+  int errno_val;
+  int host_length;
+  char* host_addr;
+};
+
+/*
+ * Only one call can be active at any given time:
+ * We expect mutual exclusion at the Scheme end
+ */
+
+static DWORD WINAPI
+gethostbyname_thread(LPVOID void_handshake)
+{
+  struct gethostbyname_handshake *handshake = void_handshake;
+  struct addrinfo *address_info;
+  struct addrinfo hints;
+  hints.ai_family = PF_INET;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_protocol = IPPROTO_TCP;
+  hints.ai_flags = AI_CANONNAME;
+  hints.ai_addrlen = 0;
+  hints.ai_canonname = NULL;
+  hints.ai_addr = NULL;
+  hints.ai_next = NULL;
+  /* ####
+   * Unicode note: GetAddrInfoW exists, but Mike doesn't know
+   * if just passing in IDNA (which we'll need to do eventually
+   * anyway) works aas well.
+   */
+  handshake->errno_val = getaddrinfo(handshake->host_name,
+				     NULL, /* servname */
+				     &hints,
+				     &address_info);
+  if (handshake->errno_val == 0)
+    {
+      handshake->host_length = sizeof(struct in_addr);
+      handshake->host_addr = malloc(handshake->host_length);
+      if (handshake->host_addr == NULL)
+	handshake->errno_val = ERROR_NOT_ENOUGH_MEMORY;
+      else
+	handshake->errno_val = 0;
+      memcpy(handshake->host_addr,
+	     (void *)&(((struct sockaddr_in*)address_info->ai_addr)->sin_addr),
+	     handshake->host_length);
+    }
+  freeaddrinfo(address_info);
+  s48_note_external_event((long) handshake->event_uid);
+  ExitThread(0);
+  return 0;
+}
+
+static s48_value
+s48_get_host_by_name(s48_value machine)
+{
+  DWORD id;
+  HANDLE thread_handle;
+  struct gethostbyname_handshake* handshake;
+  
+  int machine_length = S48_BYTE_VECTOR_LENGTH(machine);
+  if (machine_length > S48_MAX_HOST_NAME_LENGTH)
+    s48_raise_argument_type_error(machine);
+
+  handshake = malloc(sizeof(struct gethostbyname_handshake));
+  if (handshake == NULL)
+    s48_raise_out_of_memory_error();
+
+  memcpy(handshake->host_name, s48_extract_byte_vector(machine), machine_length);
+  handshake->host_name[machine_length] = '\0';
+
+  handshake->event_uid = s48_external_event_uid();
+  thread_handle = CreateThread(NULL, /* lpThreadAttributes */
+			       4096, /* dwStackSize */
+			       (LPTHREAD_START_ROUTINE)gethostbyname_thread,
+			       (LPVOID) handshake, /* lpParameter */
+			       0, /* dwCreationFlags, */
+			       &id);
+
+  {
+    s48_value sch_event_uid = S48_UNSPECIFIC;
+    s48_value sch_handshake = S48_UNSPECIFIC;
+    s48_value sch_result;
+    S48_DECLARE_GC_PROTECT(2);
+    
+    S48_GC_PROTECT_2(sch_event_uid, sch_handshake);
+    sch_event_uid = s48_enter_integer(handshake->event_uid);
+    sch_handshake = s48_enter_pointer(handshake);
+    
+    sch_result = s48_cons(sch_event_uid, sch_handshake);
+    S48_GC_UNPROTECT();
+    return sch_result;
+  }
+}
+
+static s48_value
+s48_get_host_by_name_result(s48_value sch_handshake)
+{
+  struct gethostbyname_handshake* handshake
+    = s48_extract_pointer(sch_handshake);
+
+  s48_unregister_external_event_uid(handshake->event_uid);
+
+  if (handshake->errno_val)
+    {
+      int errno_val = handshake->errno_val;
+      free(handshake);
+      s48_raise_os_error(errno_val);
+      return S48_UNSPECIFIC; /* make Visual C++ happy */
+    }
+  else
+    {
+      s48_value sch_result
+	= s48_enter_byte_vector(handshake->host_addr, handshake->host_length);
+      free(handshake->host_addr);
+      free(handshake);
+      return sch_result;
+    }
+}
+
+/*
+ * Get the name of the `sch_addr''s host.  If we can't get the real
+ * host name we use the numbers-and-dots representation of the
+ * address.
+ */
+
+struct gethostbyaddr_handshake
+{
+  long event_uid;
+  struct in_addr address;
+  char host_name[S48_MAX_HOST_NAME_LENGTH + 1];
+  int errno_val;
+};
+
+static DWORD WINAPI
+gethostbyaddr_thread(LPVOID void_handshake)
+{
+  struct gethostbyaddr_handshake *handshake = void_handshake;
+  struct sockaddr_in address;
+  memset(&address, 0, sizeof(address));
+  address.sin_family = AF_INET;
+  address.sin_addr = handshake->address;
+
+  handshake->errno_val
+    = getnameinfo((const struct sockaddr *)&address,
+		  sizeof(address),
+		  handshake->host_name, S48_MAX_HOST_NAME_LENGTH,
+		  NULL, 0, 0);
+
+  s48_note_external_event((long) handshake->event_uid);
+  ExitThread(0);
+  return 0;
+}
+ 
+static s48_value
+s48_get_host_by_address(s48_value sch_addr)
+{
+  DWORD id;
+  HANDLE thread_handle;
+  struct gethostbyaddr_handshake* handshake;
+
+  handshake = malloc(sizeof(struct gethostbyaddr_handshake));
+  if (handshake == NULL)
+    s48_raise_out_of_memory_error();
+
+  memcpy(&(handshake->address),
+	 (char *) s48_extract_byte_vector(sch_addr), sizeof(struct in_addr));
+
+  handshake->event_uid = s48_external_event_uid();
+
+  thread_handle = CreateThread(NULL, /* lpThreadAttributes */
+			       4096, /* dwStackSize */
+			       (LPTHREAD_START_ROUTINE)gethostbyaddr_thread,
+			       handshake, /* lpParameter */
+			       0, /* dwCreationFlags, */
+			       &id);
+  
+  {
+    s48_value sch_event_uid = S48_UNSPECIFIC;
+    s48_value sch_handshake = S48_UNSPECIFIC;
+    s48_value sch_result;
+    S48_DECLARE_GC_PROTECT(2);
+    
+    S48_GC_PROTECT_2(sch_event_uid, sch_handshake);
+    sch_event_uid = s48_enter_integer(handshake->event_uid);
+    sch_handshake = s48_enter_pointer(handshake);
+    
+    sch_result = s48_cons(sch_event_uid, sch_handshake);
+    S48_GC_UNPROTECT();
+    return sch_result;
+  }
+}
+	 
+static s48_value 
+s48_get_host_by_address_result(s48_value sch_handshake)
+{
+  struct gethostbyaddr_handshake* handshake
+    = s48_extract_pointer(sch_handshake);
+  s48_value sch_result;
+
+  s48_unregister_external_event_uid(handshake->event_uid);
+  if (handshake->errno_val)
+    {
+      int errno_val = handshake->errno_val;
+      free(handshake);
+      s48_raise_os_error(errno_val);
+      return S48_UNSPECIFIC; /* make Visual C++ happy */
+    }
+  else
+    {
+      sch_result = s48_enter_string_latin_1(handshake->host_name);
+      free(handshake);
+      return sch_result;
+    }
+}
+
 /*
  * Create an internet-domain stream (reliable, sequenced) socket.
  * We return an input channel on success and raise an exception on failure.
@@ -1676,14 +1910,12 @@ connect_callback(DWORD dwParam)
 
 static s48_value
 s48_connect(s48_value channel,
-	    s48_value machine,
+	    s48_value sch_address,
 	    s48_value port,
 	    s48_value retry_p)
 {
   long socket_fd;
   unsigned short port_number;
-  char *machine_name;
-  struct addrinfo *address_info;
   struct sockaddr_in address;
   stream_descriptor_t* stream_descriptor;
   callback_data_t* callback_data;
@@ -1696,50 +1928,20 @@ s48_connect(s48_value channel,
   callback_data = &(stream_descriptor->callback_data);
   socket = stream_descriptor->socket_data.socket;
 
-  machine_name = s48_extract_byte_vector(machine);
-  
   S48_CHECK_FIXNUM(port);
   port_number = (unsigned short)S48_UNSAFE_EXTRACT_FIXNUM(port);
-  
-  /*
-   * Get the host and initialize `address'.
-   * ####Note that this blocks.
-   */
-
-  {
-      struct addrinfo hints;
-      int status;
-      hints.ai_family = PF_INET;
-      hints.ai_socktype = SOCK_STREAM;
-      hints.ai_protocol = IPPROTO_TCP;
-      hints.ai_flags = AI_CANONNAME;
-      hints.ai_addrlen = 0;
-      hints.ai_canonname = NULL;
-      hints.ai_addr = NULL;
-      hints.ai_next = NULL;
-      /* ####
-       * Unicode note: GetAddrInfoW exists, but Mike doesn't know
-       * if just passing in IDNA (which we'll need to do eventually
-       * anyway) works aas well.
-       */
-      status = getaddrinfo(machine_name,
-			   NULL, /* servname */
-			   &hints,
-			   &address_info);
-      if (status != 0)
-	raise_windows_error(status);
-  }
 
   memset((void *)&address, 0, sizeof(address));
+  address.sin_family = AF_INET;
 
-  if (address_info->ai_addrlen > sizeof(address))
-    s48_raise_range_error(s48_enter_fixnum(address_info->ai_addrlen),
+  if (S48_BYTE_VECTOR_LENGTH(sch_address) > sizeof(address))
+    s48_raise_range_error(s48_enter_fixnum(S48_BYTE_VECTOR_LENGTH(sch_address)),
 			  S48_UNSAFE_ENTER_FIXNUM(0),
 			  s48_enter_fixnum(sizeof(address)));
 
-  memcpy((void *)&address, (void *)address_info->ai_addr, address_info->ai_addrlen);
-  freeaddrinfo(address_info);
-
+  memcpy((void *)&address.sin_addr,
+	 (void *)s48_extract_byte_vector(sch_address),
+	 S48_BYTE_VECTOR_LENGTH(sch_address));
   address.sin_port = htons(port_number);
 
   /*
