@@ -1,4 +1,4 @@
-; Copyright (c) 1993-2007 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 
 ; Some interfaces.  Order of presentation is a bit random.
@@ -601,7 +601,8 @@
   (export os-string?
 	  string->os-string byte-vector->os-string x->os-string
 	  os-string->string os-string->byte-vector
-	  call-with-os-string-text-codec))
+	  call-with-os-string-text-codec
+	  current-os-string-text-codec))
 
 (define-interface i/o-internal-interface
   (export input-port-option		;read.scm
@@ -729,7 +730,7 @@
 	  round-robin-event-handler
 
 	  make-counter            ; for thread counts
-	  counter-value
+	  counter-value set-counter!
           increment-counter!
           decrement-counter!))
 
@@ -773,6 +774,11 @@
 	  call-before-heap-overflow!
 	  (interrupt :syntax)))
 
+(define-interface external-events-interface
+  (export initialize-external-events!	;init.scm
+	  waiting-for-external-events?  ;root-scheduler.scm
+	  wait-for-external-event))
+
 (define-interface writing-interface
   (export write
 	  display
@@ -861,7 +867,9 @@
 	  file-name-directory
 	  file-name-nondirectory
 	  translate
+	  set-global-translation!
 	  set-translation!
+	  make-translations with-translations
 	  translations))
 
 ; Things for the compiler.
@@ -970,11 +978,11 @@
 
 	  make-name-table
 
+	  make-qualified
 	  qualified?
 	  qualified-parent-name
 	  qualified-symbol
 	  qualified-uid
-	  name->qualified
 
           desyntaxify
 	  ))
@@ -1050,6 +1058,8 @@
 	  force-node
 	  schemify
 
+	  name->qualified
+
 	  get-operator
 	  make-operator-table
 	  operator-name
@@ -1237,6 +1247,7 @@
 	  location-info-table		;debuginfo, disclosers
 	  package-definition
 	  package-unstable?		;env/pacman.scm
+	  environment-stable?           ;comp/schemify.scm
 	  package-integrate?		;env/debug.scm
 	  package-add-static!		;opt/analyze.scm
 	  package-refine-type!		;opt/usage.scm
