@@ -1,4 +1,4 @@
-; Copyright (c) 1993-2007 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
 
 
 ; More and more packages.  Some of these get loaded into the initial
@@ -104,7 +104,7 @@
   (optimize auto-integrate))
 
 (define-structure locks locks-interface
-  (open scheme-level-1 queues
+  (open scheme-level-2 queues
 	threads threads-internal
 	interrupts
 	proposals)
@@ -316,19 +316,10 @@
   (files (big import-def)
 	 (big callback)))
 
-(define-structure dynamic-externals dynamic-externals-interface
-  (open scheme-level-2 define-record-types tables
-        signals          	;warn
-	primitives		;find-all-records
-	i/o			;current-error-port
-        code-vectors
-	os-strings text-codecs
-	external-calls)
-  (files (big external)))
-
 (define-structure shared-objects shared-objects-interface
   (open scheme-level-2
 	define-record-types
+	exceptions
 	external-calls
 	os-strings text-codecs)
   (files (big shared-object)))
@@ -339,6 +330,7 @@
 	shared-objects
 	(subset usual-resumer (add-initialization-thunk!))
 	(subset big-util (delq delete any))
+	filenames
 	(subset signals (error)))
   (files (big dynamic-external)))
 
@@ -390,6 +382,8 @@
 				     socket-port-number
 				     socket-client
 				     get-host-name
+				     get-host-by-name
+				     get-host-by-address
 
 				     ; From the old interface
 				     ; I would like to get rid of these.
@@ -416,6 +410,7 @@
 	channel-ports		; {in|out}put-channel->port
 	channel-i/o		; wait-for-channel
 	condvars		; for wait-for-channel
+	external-events
 	byte-vectors)
   (files (big socket)))
 
@@ -522,31 +517,6 @@
 (define-structure bigbit (export)
   (open scheme-level-2))
 
-; Externals - this is obsolete; use external-calls and dynamic-externals
-; instead.
-
-(define-structure externals (compound-interface
-			       dynamic-externals-interface
-			       (export external-call
-				       null-terminate))
-  (open scheme-level-2 dynamic-externals
-	(subset external-calls (import-lambda-definition)))
-  (begin
-   ; We fake the old external-call primitive using the new one and a
-   ; a C helper procedure from c/unix/dynamo.c.
-
-    (define (external-call proc . args)
-      (let ((args (apply vector args)))
-	(old-external-call (external-value proc) args)))
-    
-    (import-lambda-definition old-external-call
-			      (proc args)
-			      "s48_old_external_call")
-
-    ; All strings are now null terminated.
-    (define (null-terminate string) string)))
-
-;----------------
 ; ... end of package definitions.
 
 ; Temporary compatibility stuff
